//  Demo 17: ThreadPracticeD.java
//
//   Another (More Complex) Java Application
//   Using the MovingSquareThread Class.
//   
//   This time, a linked list of Threads!
//
//   Graphically demonstrates separate Threads in action.
//
//             by Charles W. Neville
//      Copyright Charles W. Neville, July, 1999

// Like just like ThreadPracticeC.java, but uses a linked list 
// of Threads so the Java garbage collector will not delete 
// unreferenced MovingSquareThreads.

// Threads:
//
//   1. Main Thread -- this application.  Every running Java program
//        contains at least one Thread, the Thread main() runs on.
//   2. A separate Thread for each moving Square.  The moving Squares
//        are actually separate MovingSquareThread objects, launched
//        by invoking their library start() method.  Their start()
//        is inherited from the Thread class.  The start() method 
//        sets the new Threads up to run and then calls their run() 
//        method.  Their run() method is where the animation code 
//        goes.
      
import java.awt.*;
import java.awt.event.*;
import CloseAndExit;
import MovingSquareThread;
import MovingSquareThreadList;

public class ThreadPracticeD extends Frame 
                             implements ActionListener {

    MovingSquareThreadList mstList;  
                 // A linked list of MovingSquareThreads.  We
                 // will add new MovingSquareThread objects to
                 // the list when we create them.  If we just
                 // reassigned new MovingSquareThread objects 
                 // to a MovingSquareThread variable, the Java
                 // garbage collector would immediately delete
                 // the old objects

    Image imageBuffer;    // for graphics double buffering as usual
 
    // for new Square objects
    Label xLabel, yLabel,    // Labels for x, y components of 
          sizeLabel,         // position, size and color
          colorLabel;

    TextField xInput,        // TextFields for same
              yInput,
              sizeInput,
              colorInput;

    // for new MovingSquareThread objects 
    Label rxLabel, ryLabel,  // Labels for x, y components of motion 
          dLabel;            // vector, and distance

    TextField rxInput,       // TextFields for same
              ryInput,
              dInput; 

    Button launchButton;     // launch a new MovingSquareThread

    int x, y, size;  // x and y components of new Square postion,
                     // plus size of new Square

    Color color;     // Color of new Square

    int rx, ry, d;   // x and y components of motion vector, plus
                     // distance, in units of the motion vector
                     // length

    // Constructor: builds Frame and initializes variables.
    public ThreadPracticeD() {

       setLayout(new FlowLayout());

       // for new Square objects
       xLabel = new Label(" x Coord ");
       xInput = new TextField(4);
       xInput.setText("50");
       add(xLabel);  add(xInput);

       yLabel = new Label(" y Coord ");
       yInput = new TextField(4);
       yInput.setText("100");
       add(yLabel);  add(yInput);

       sizeLabel = new Label(" Size ");
       sizeInput = new TextField(4);
       sizeInput.setText("25");
       add(sizeLabel);  add(sizeInput);

       colorLabel = new Label("Color");
       colorInput = new TextField(5);
       colorInput.setText("blue");
       add(colorLabel);  add(colorInput);

       // for new MovingSquareThread objects
       rxLabel = new Label(" x Motion ");
       rxInput = new TextField(3);
       rxInput.setText("1");
       add(rxLabel);  add(rxInput);

       ryLabel = new Label(" y Motion ");
       ryInput = new TextField(3);
       ryInput.setText("1");
       add(ryLabel);  add(ryInput);

       dLabel = new Label(" Distance ");
       dInput = new TextField(4);
       dInput.setText("200");
       add(dLabel);  add(dInput);

       launchButton = new Button("Launch");
       launchButton.addActionListener(this);
       add(launchButton);

       x = 50;  y = 100; size = 25; color = Color.blue;
       rx = 1; ry = 1;   d = 200;
 
       addWindowListener(new CloseAndExit());   // as usual

    } // Constructor ThreadPracticeD()

    public void actionPerformed(ActionEvent evt) {

      MovingSquareThread mst;

      MovingSquareThreadList first;  // new first node in linked
                                     // list of MovingSquareThread
                                     // objects

      String colorString;

      try {
        x = Integer.parseInt(xInput.getText());
      }
      catch (NumberFormatException e) {
        xInput.setText(Integer.toString(x)); // re-establish former
                                             // value
      }
 
      try {
        y = Integer.parseInt(yInput.getText());
      }
      catch (NumberFormatException e) {
        yInput.setText(Integer.toString(y)); // re-establish former
                                             // value
      }

      try {
        size = Integer.parseInt(sizeInput.getText());
      }
      catch (NumberFormatException e) {
        sizeInput.setText(Integer.toString(size)); 
                                      // re-establish former value
      }

      colorString = colorInput.getText();
      // always use the equals() method of the String class to
      // compare Strings for equality.  The == operator does object
      // comparison, that is it compares addresses, not content.
      if (colorString.equals("red"))
        color = Color.red;
      else if (colorString.equals("white"))
        color = Color.white;
      else if (colorString.equals("blue"))
        color = Color.blue;
      else if (colorString.equals("green"))
        color = Color.green;
      else { // restore default
        color = Color.blue;
        colorInput.setText("blue");
      }

      try {
        rx = Integer.parseInt(rxInput.getText());
      }
      catch (NumberFormatException e) {
        rxInput.setText(Integer.toString(rx)); // re-establish
                                               // former value
      }

      try {
        ry = Integer.parseInt(ryInput.getText());
      }
      catch (NumberFormatException e) {
        ryInput.setText(Integer.toString(ry)); // re-establish
                                               // former value
      }

      try {
        d = Integer.parseInt(dInput.getText());
      }
      catch (NumberFormatException e) {
        dInput.setText(Integer.toString(d)); // re-establish former
                                             // value
      }

      mst = new MovingSquareThread( new Square(x, y, size, 
                    color), this, imageBuffer, rx, ry, d);

      first = new MovingSquareThreadList(mst);

      if (mstList == null) mstList = first;
      else mstList = mstList.addToFront(first);


      mstList.mst.start();

    } // actionPerformed()

   // init(): creates the Image buffer object.  Remember, we can't
   //         do this in our Constructor, because we actually have
   //         to exist before the createImage method will work.
   //         We have to remember to call this method from main(),
   //         or we will get the dreaded "pointer reference to
   //         null" Exception.
    public void init() {

       imageBuffer = createImage(400, 400);

       while(imageBuffer == null) {

          try {
             Thread.sleep(50);  // 50 millisecond pause
          }
          catch(InterruptedException e) {
             // do nothing
          }

       }

    } // init()

    public static void main(String[ ] args) {

       ThreadPracticeD tpd = new ThreadPracticeD();
       tpd.setSize(400, 400);
       tpd.setVisible(true);
       tpd.init();  // don't forget to call this!

    } // main()

} // class ThreadPracticeD